@@ -1,4 +1,3 @@
-#!/usr/bin/env python2
 # Terminator by Chris Jones <cmsj@tenshu.net>
 # GPL v2 only
 """plugin.py - Base plugin system
@@ -8,9 +7,10 @@
    considered BSD licenced, per the authors wishes)
 
 >>> registry = PluginRegistry()
->>> registry.instances
-{}
->>> registry.load_plugins(True)
+>>> isinstance(registry.instances, dict)
+True
+>>> registry.enable('TestPlugin')
+>>> registry.load_plugins()
 >>> plugins = registry.get_plugins_by_capability('test')
 >>> len(plugins)
 1
@@ -25,10 +25,10 @@
 
 import sys
 import os
-import borg
-from config import Config
-from util import dbg, err, get_config_dir
-from terminator import Terminator
+from . import borg
+from .config import Config
+from .util import dbg, err, get_config_dir
+from .terminator import Terminator
 
 class Plugin(object):
     """Definition of our base plugin class"""
@@ -70,7 +70,7 @@ class PluginRegistry(borg.Borg):
         if not self.available_plugins:
             self.available_plugins = {}
 
-    def load_plugins(self, testing=False):
+    def load_plugins(self):
         """Load all plugins present in the plugins/ directory in our module"""
         if self.done:
             dbg('PluginRegistry::load_plugins: Already loaded')
@@ -95,16 +95,16 @@ class PluginRegistry(borg.Borg):
                     try:
                         module = __import__(plugin[:-3], None, None, [''])
                         for item in getattr(module, 'AVAILABLE'):
-                            if item not in self.available_plugins.keys():
+                            if item not in list(self.available_plugins.keys()):
                                 func = getattr(module, item)
                                 self.available_plugins[item] = func
 
-                            if not testing and item not in config['enabled_plugins']:
+                            if item not in config['enabled_plugins']:
                                 dbg('plugin %s not enabled, skipping' % item)
                                 continue
                             if item not in self.instances:
                                 self.instances[item] = func()
-                    except Exception, ex:
+                    except Exception as ex:
                         err('PluginRegistry::load_plugins: Importing plugin %s \
 failed: %s' % (plugin, ex))
 
@@ -127,12 +127,12 @@ for %s' % (len(self.instances), capability))
     def get_available_plugins(self):
         """Return a list of all available plugins whether they are enabled or
         disabled"""
-        return(self.available_plugins.keys())
+        return(list(self.available_plugins.keys()))
 
     def is_enabled(self, plugin):
         """Return a boolean value indicating whether a plugin is enabled or
         not"""
-        return(self.instances.has_key(plugin))
+        return(plugin in self.instances)
 
     def enable(self, plugin):
         """Enable a plugin"""
@@ -173,7 +173,7 @@ class URLHandler(Plugin):
 
     def unload(self):
         """Handle being removed"""
-        if not self.match:
+        if not self.handler_name:
             err('unload called without self.handler_name being set')
             return
         terminator = Terminator()
