@@ -1,4 +1,4 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 """Preferences Editor for Terminator. 
 
 Load a UIBuilder config file, display it,
@@ -10,20 +10,27 @@ write it to a config file
 import os
 from gi.repository import GObject, Gtk, Gdk
 
-from util import dbg, err
-import config
-from keybindings import Keybindings, KeymapError
-from translation import _
-from encoding import TerminatorEncoding
-from terminator import Terminator
-from plugin import PluginRegistry
-from version import APP_NAME
+from .util import dbg, err
+from . import config
+from .keybindings import Keybindings, KeymapError
+from .translation import _
+from .encoding import TerminatorEncoding
+from .terminator import Terminator
+from .plugin import PluginRegistry
+from .version import APP_NAME
+
+def get_color_string(widcol):
+    return('#%02x%02x%02x' % (widcol.red>>8, widcol.green>>8, widcol.blue>>8))
 
 def color2hex(widget):
     """Pull the colour values out of a Gtk ColorPicker widget and return them
     as 8bit hex values, sinces its default behaviour is to give 16bit values"""
-    widcol = widget.get_color()
-    return('#%02x%02x%02x' % (widcol.red>>8, widcol.green>>8, widcol.blue>>8))
+    return get_color_string(widget.get_color())
+
+def rgba2hex(widget):
+    return get_color_string(widget.get_rgba().to_color())
+
+NUM_PALETTE_COLORS = 16
 
 # FIXME: We need to check that we have represented all of Config() below
 class PrefsEditor:
@@ -33,6 +40,8 @@ class PrefsEditor:
     plugins = None
     keybindings = None
     window = None
+    calling_window = None
+    term = None
     builder = None
     layouteditor = None
     previous_layout_selection = None
@@ -96,6 +105,9 @@ class PrefsEditor:
     keybindingnames = { 'zoom_in'          : _('Increase font size'),
                         'zoom_out'         : _('Decrease font size'),
                         'zoom_normal'      : _('Restore original font size'),
+						'zoom_in_all'	   : _('Increase font size on all terminals'),
+						'zoom_out_all'	   : _('Decrease font size on all terminals'),
+						'zoom_normal_all'  : _('Restore original font size on all terminals'),
                         'new_tab'          : _('Create a new tab'),
                         'cycle_next'       : _('Focus the next terminal'),
                         'cycle_prev'       : _('Focus the previous terminal'),
@@ -145,6 +157,7 @@ class PrefsEditor:
                         'reset'            : _('Reset the terminal'),
                         'reset_clear'      : _('Reset and clear the terminal'),
                         'hide_window'      : _('Toggle window visibility'),
+                        'create_group'     : _('Create new group'),
                         'group_all'        : _('Group all terminals'),
                         'group_all_toggle' : _('Group/Ungroup all terminals'),
                         'ungroup_all'      : _('Ungroup all terminals'),
@@ -164,6 +177,7 @@ class PrefsEditor:
                         'layout_launcher'  : _('Open layout launcher window'),
                         'next_profile'     : _('Switch to next profile'),
                         'previous_profile' : _('Switch to previous profile'), 
+			'preferences'	   : _('Open the Preferences window'),
                         'help'             : _('Open the manual')
             }
 
@@ -171,18 +185,21 @@ class PrefsEditor:
         self.config = config.Config()
         self.config.base.reload()
         self.term = term
+        self.calling_window = self.term.get_toplevel()
+        self.calling_window.preventHide = True
         self.builder = Gtk.Builder()
         self.builder.set_translation_domain(APP_NAME)
         self.keybindings = Keybindings()
+        self.active_message_dialog = None
         try:
             # Figure out where our library is on-disk so we can open our
             (head, _tail) = os.path.split(config.__file__)
             librarypath = os.path.join(head, 'preferences.glade')
             gladefile = open(librarypath, 'r')
             gladedata = gladefile.read()
-        except Exception, ex:
-            print "Failed to find preferences.glade"
-            print ex
+        except Exception as ex:
+            print("Failed to find preferences.glade")
+            print(ex)
             return
 
         self.builder.add_from_string(gladedata)
@@ -203,7 +220,7 @@ class PrefsEditor:
         try:
             self.config.inhibit_save()
             self.set_values()
-        except Exception, e:
+        except Exception as e:
             err('Unable to set values: %s' % e)
         self.config.uninhibit_save()
 
@@ -212,6 +229,7 @@ class PrefsEditor:
         terminator = Terminator()
         terminator.reconfigure()
         self.window.destroy()
+        self.calling_window.preventHide = False
         del(self)
 
     def set_values(self):
@@ -235,6 +253,13 @@ class PrefsEditor:
         widget.set_value(float(termsepsize))
         widget = guiget('handlesize_value_label')
         widget.set_text(str(termsepsize))
+        # Line Height
+        lineheightsize = self.config['line_height']
+        lineheightsize = round(float(lineheightsize),1)
+        widget = guiget('lineheight')
+        widget.set_value(lineheightsize)
+        widget = guiget('lineheight_value_label')
+        widget.set_text(str(lineheightsize))
         # Window geometry hints
         geomhint = self.config['geometry_hinting']
         widget = guiget('wingeomcheck')
@@ -281,6 +306,9 @@ class PrefsEditor:
         else:
             active = 1
         widget.set_active(active)
+        # Disable Ctrl+mousewheel zoom
+        widget = guiget('disablemousewheelzoom')
+        widget.set_active(self.config['disable_mousewheel_zoom'])
         # scroll_tabbar
         widget = guiget('scrolltabbarcheck')
         widget.set_active(self.config['scroll_tabbar'])
@@ -305,12 +333,23 @@ class PrefsEditor:
         #Hide size text from the title bar
         widget = guiget('title_hide_sizetextcheck')
         widget.set_active(self.config['title_hide_sizetext'])
+
+        # title bar at bottom
+        widget = guiget('title_at_bottom_checkbutton')
+        widget.set_active(self.config['title_at_bottom'])
+        
         #Always split with profile
         widget = guiget('always_split_with_profile')
         widget.set_active(self.config['always_split_with_profile'])
         # Putty paste style
         widget = guiget('putty_paste_style')
         widget.set_active(self.config['putty_paste_style'])
+        # Putty paste style source clipboard
+        if self.config['putty_paste_style_source_clipboard']:
+            widget = guiget('putty_paste_style_source_clipboard_radiobutton')
+        else:
+            widget = guiget('putty_paste_style_source_primary_radiobutton')
+        widget.set_active(True)
         # Smart copy
         widget = guiget('smart_copy')
         widget.set_active(self.config['smart_copy'])
@@ -446,9 +485,6 @@ class PrefsEditor:
         # Copy on selection
         widget = guiget('copy_on_selection')
         widget.set_active(self.config['copy_on_selection'])
-        # Rewrap on resize
-        widget = guiget('rewrap_on_resize_checkbutton')
-        widget.set_active(self.config['rewrap_on_resize'])
         # Word chars
         widget = guiget('word_chars_entry')
         widget.set_text(self.config['word_chars'])
@@ -511,6 +547,9 @@ class PrefsEditor:
         # Use system colors
         widget = guiget('use_theme_colors_checkbutton')
         widget.set_active(self.config['use_theme_colors'])
+        # Bold is bright
+        widget = guiget('bold_text_is_bright_checkbutton')
+        widget.set_active(self.config['bold_is_bright'])
         # Colorscheme
         widget = guiget('color_scheme_combobox')
         scheme = None
@@ -529,15 +568,16 @@ class PrefsEditor:
                 scheme = 'custom'
         # NOTE: The scheme is set in the GUI widget after the fore/back colours
         # Foreground color
-        widget = guiget('foreground_colorpicker')
-        widget.set_color(Gdk.color_parse(self.config['foreground_color']))
+        widget = guiget('foreground_colorbutton')
+        widget.set_events(Gdk.EventMask.BUTTON_PRESS_MASK)
+
         if scheme == 'custom':
             widget.set_sensitive(True)
         else:
             widget.set_sensitive(False)
         # Background color
-        widget = guiget('background_colorpicker')
-        widget.set_color(Gdk.color_parse(self.config['background_color']))
+        widget = guiget('background_colorbutton')
+        widget.set_events(Gdk.EventMask.BUTTON_PRESS_MASK)
         if scheme == 'custom':
             widget.set_sensitive(True)
         else:
@@ -558,10 +598,13 @@ class PrefsEditor:
                 palette = 'custom'
         # NOTE: The palette selector is set after the colour pickers
         # Palette colour pickers
-        colourpalette = self.config['palette'].split(':')
-        for i in xrange(1, 17):
-            widget = guiget('palette_colorpicker_%d' % i)
-            widget.set_color(Gdk.color_parse(colourpalette[i - 1]))
+        for palette_id in range(0, NUM_PALETTE_COLORS):
+            widget = self.get_palette_widget(palette_id)
+            widget.set_events(Gdk.EventMask.BUTTON_PRESS_MASK)
+            def on_palette_click(event, data, widget=widget):
+                self.edit_palette_button(widget)
+            widget.connect('button-press-event', on_palette_click)
+        self.load_palette()
         # Now set the palette selector widget
         widget = guiget('palette_combobox')
         widget.set_active(self.palettevalues[palette])
@@ -590,11 +633,15 @@ class PrefsEditor:
             guiget('solid_radiobutton').set_active(True)
         elif self.config['background_type'] == 'transparent':
             guiget('transparent_radiobutton').set_active(True)
+        elif self.config['background_type'] == 'image':
+            guiget('image_radiobutton').set_active(True)
         self.update_background_tab()
         # Background shading
         widget = guiget('background_darkness_scale')
         widget.set_value(float(self.config['background_darkness']))
-
+        widget = guiget('background_image_file')
+        widget.set_filename(self.config['background_image'])
+   
         ## Scrolling tab
         # Scrollbar position
         widget = guiget('scrollbar_position_combobox')
@@ -647,7 +694,7 @@ class PrefsEditor:
         encodingstore = guiget('EncodingListStore')
         value = self.config['encoding']
         encodings = TerminatorEncoding().get_list()
-        encodings.sort(lambda x, y: cmp(x[2].lower(), y[2].lower()))
+        encodings.sort(key=lambda x: x[2].lower())
 
         for encoding in encodings:
             if encoding[1] is None:
@@ -690,6 +737,11 @@ class PrefsEditor:
         self.config['dbus'] = widget.get_active()
         self.config.save()
 
+    def on_disable_mousewheel_zoom_toggled(self, widget):
+        """Ctrl+mousewheel zoom setting changed"""
+        self.config['disable_mousewheel_zoom'] = widget.get_active()
+        self.config.save()
+
     def on_winbordercheck_toggled(self, widget):
         """Window border setting changed"""
         self.config['borderless'] = not widget.get_active()
@@ -725,6 +777,11 @@ class PrefsEditor:
         self.config['title_hide_sizetext'] = widget.get_active()
         self.config.save()
 
+    def on_title_at_bottom_checkbutton_toggled(self, widget):
+        """Title at bottom setting changed"""
+        self.config['title_at_bottom'] = widget.get_active()
+        self.config.save()
+
     def on_always_split_with_profile_toggled(self, widget):
         """Always split with profile setting changed"""
         self.config['always_split_with_profile'] = widget.get_active()
@@ -745,21 +802,28 @@ class PrefsEditor:
         self.config['copy_on_selection'] = widget.get_active()
         self.config.save()
 
-    def on_rewrap_on_resize_toggled(self, widget):
-        """Rewrap on resize setting changed"""
-        self.config['rewrap_on_resize'] = widget.get_active()
-        self.config.save()
-
     def on_putty_paste_style_toggled(self, widget):
         """Putty paste style setting changed"""
         self.config['putty_paste_style'] = widget.get_active()
         self.config.save()
 
+    def on_putty_paste_style_source_clipboard_toggled(self, widget):
+        """PuTTY paste style source changed"""
+        guiget = self.builder.get_object
+        clipboardwidget = guiget('putty_paste_style_source_clipboard_radiobutton')
+        self.config['putty_paste_style_source_clipboard'] = clipboardwidget.get_active()
+        self.config.save()
+
     def on_smart_copy_toggled(self, widget):
         """Putty paste style setting changed"""
         self.config['smart_copy'] = widget.get_active()
         self.config.save()
 
+    def on_clear_select_on_copy_toggled(self,widget):
+        """Clear selection on smart copy"""
+        self.config['clear_select_on_copy'] = widget.get_active()
+        self.config.save()
+
     def on_cursor_blink_toggled(self, widget):
         """Cursor blink setting changed"""
         self.config['cursor_blink'] = widget.get_active()
@@ -827,7 +891,7 @@ class PrefsEditor:
         elif selected == 2:
             value = 'ascii-del'
         elif selected == 3:
-            value == 'escape-sequence'
+            value = 'escape-sequence'
         else:
             value = 'automatic'
         self.config['backspace_binding'] = value
@@ -871,6 +935,11 @@ class PrefsEditor:
         self.config['scrollbar_position'] = value
         self.config.save()
 
+    def on_background_image_file_set(self,widget):
+        print(widget.get_filename())
+        self.config['background_image'] = widget.get_filename()
+        self.config.save()
+
     def on_darken_background_scale_value_changed(self, widget):
         """Background darkness setting changed"""
         value = widget.get_value()  # This one is rounded according to the UI.
@@ -882,34 +951,28 @@ class PrefsEditor:
     def on_palette_combobox_changed(self, widget):
         """Palette selector changed"""
         value = None
-        guiget = self.builder.get_object
         active = widget.get_active()
 
-        for key in self.palettevalues.keys():
+        for key in list(self.palettevalues.keys()):
             if self.palettevalues[key] == active:
                 value = key
 
-        if value == 'custom':
-            sensitive = True
-        else:
-            sensitive = False
-
-        for num in xrange(1, 17):
-            picker = guiget('palette_colorpicker_%d' % num)
-            picker.set_sensitive(sensitive)
+        sensitive = value == 'custom'
+        for palette_id in range(0, NUM_PALETTE_COLORS):
+            self.get_palette_widget(palette_id).set_sensitive(sensitive)
 
         if value in self.palettes:
             palette = self.palettes[value]
             palettebits = palette.split(':')
-            for num in xrange(1, 17):
+            for palette_id in range(0, NUM_PALETTE_COLORS):
                 # Update the visible elements
-                picker = guiget('palette_colorpicker_%d' % num)
-                picker.set_color(Gdk.color_parse(palettebits[num - 1]))
+                color = Gdk.color_parse(palettebits[palette_id])
+                self.load_palette_color(palette_id, color)
         elif value == 'custom':
             palettebits = []
-            for num in xrange(1, 17):
-                picker = guiget('palette_colorpicker_%d' % num)
-                palettebits.append(color2hex(picker))
+            for palette_id in range(0, NUM_PALETTE_COLORS):
+                # Save the custom values into the configuration.
+                palettebits.append(get_color_string(self.get_palette_color(palette_id)))
             palette = ':'.join(palettebits)
         else:
             err('Unknown palette value: %s' % value)
@@ -918,32 +981,148 @@ class PrefsEditor:
         self.config['palette'] = palette
         self.config.save()
 
-    def on_background_colorpicker_color_set(self, widget):
-        """Background color changed"""
-        self.config['background_color'] = color2hex(widget)
-        self.config.save()
+    def on_foreground_colorbutton_draw(self, widget, cr):
+        width = widget.get_allocated_width()
+        height = widget.get_allocated_height()
+        col = Gdk.color_parse(self.config['foreground_color'])
+        cr.rectangle(0, 0, width, height)
+        cr.set_source_rgba(0.7, 0.7, 0.7, 1)
+        cr.fill()
+        cr.rectangle(1, 1, width-2, height-2)
+        cr.set_source_rgba(col.red_float, col.green_float, col.blue_float)
+        cr.fill()
 
-    def on_foreground_colorpicker_color_set(self, widget):
+    def on_foreground_colorbutton_click(self, event, data):
+        dialog = Gtk.ColorChooserDialog("Choose Terminal Text Color")
+        fg = self.config['foreground_color']
+        dialog.set_rgba(Gdk.RGBA.from_color(Gdk.color_parse(self.config['foreground_color'])))
+        dialog.connect('notify::rgba', self.on_foreground_colorpicker_color_change)
+        res = dialog.run()
+        if res != Gtk.ResponseType.OK:
+            self.config['foreground_color'] = fg
+            self.config.save()
+            terminator = Terminator()
+            terminator.reconfigure()
+        dialog.destroy()
+
+    def on_foreground_colorpicker_color_change(self, widget, color):
         """Foreground color changed"""
-        self.config['foreground_color'] = color2hex(widget)
+        self.config['foreground_color'] = rgba2hex(widget)
         self.config.save()
+        terminator = Terminator()
+        terminator.reconfigure()
 
-    def on_palette_colorpicker_color_set(self, widget):
-        """A palette colour changed"""
-        palette = None
-        palettebits = []
+    def on_background_colorbutton_draw(self, widget, cr):
+        width = widget.get_allocated_width()
+        height = widget.get_allocated_height()
+        col = Gdk.color_parse(self.config['background_color'])
+        cr.rectangle(0, 0, width, height)
+        cr.set_source_rgba(0.7, 0.7, 0.7, 1)
+        cr.fill()
+        cr.rectangle(1, 1, width-2, height-2)
+        cr.set_source_rgba(col.red_float, col.green_float, col.blue_float)
+        cr.fill()
+
+    def on_background_colorbutton_click(self, event, data):
+        dialog = Gtk.ColorChooserDialog("Choose Terminal Background Color")
+        orig = self.config['background_color']
+        dialog.connect('notify::rgba', self.on_background_colorpicker_color_change)
+        dialog.set_rgba(Gdk.RGBA.from_color(Gdk.color_parse(orig)))
+        res = dialog.run()
+        if res != Gtk.ResponseType.OK:
+            self.config['background_color'] = orig
+            self.config.save()
+            terminator = Terminator()
+            terminator.reconfigure()
+        dialog.destroy()
+
+    def on_background_colorpicker_color_change(self, widget, color):
+        """Background color changed"""
+        self.config['background_color'] = rgba2hex(widget)
+        self.config.save()
+        terminator = Terminator()
+        terminator.reconfigure()
+
+    def get_palette_widget(self, palette_id):
+        """Returns the palette widget for the given palette ID."""
         guiget = self.builder.get_object
+        return guiget('palette_colorpicker_%d' % (palette_id + 1))
 
-        # FIXME: We do this at least once elsewhere. refactor!
-        for num in xrange(1, 17):
-            picker = guiget('palette_colorpicker_%d' % num)
-            value = color2hex(picker)
-            palettebits.append(value)
-        palette = ':'.join(palettebits)
+    def get_palette_id(self, widget):
+        """Returns the palette ID for the given palette widget."""
+        for palette_id in range(0, NUM_PALETTE_COLORS):
+            if widget == self.get_palette_widget(palette_id):
+                return palette_id
+        return None
 
-        self.config['palette'] = palette
+    def get_palette_color(self, palette_id):
+        """Returns the configured Gdk color for the given palette ID."""
+        if self.config['palette'] in self.palettes:
+            colourpalette = self.palettes[self.config['palette']]
+        else:
+            colourpalette = self.config['palette'].split(':')
+        return Gdk.color_parse(colourpalette[palette_id])
+
+    def on_palette_colorpicker_draw(self, widget, cr):
+        width = widget.get_allocated_width()
+        height = widget.get_allocated_height()
+        cr.rectangle(0, 0, width, height)
+        cr.set_source_rgba(0.7, 0.7, 0.7, 1)
+        cr.fill()
+        cr.rectangle(1, 1, width-2, height-2)
+        col = self.get_palette_color(self.get_palette_id(widget))
+        cr.set_source_rgba(col.red_float, col.green_float, col.blue_float)
+        cr.fill()
+
+    def load_palette_color(self, palette_id, color):
+        """Given a palette ID and a Gdk color, load that color into the
+        specified widget."""
+        widget = self.get_palette_widget(palette_id)
+        widget.queue_draw()
+
+    def replace_palette_color(self, palette_id, color):
+        """Replace the configured palette color for the given palette ID
+        with the given color."""
+        palettebits = self.config['palette'].split(':')
+        palettebits[palette_id] = get_color_string(color)
+        self.config['palette'] = ':'.join(palettebits)
         self.config.save()
 
+    def load_palette(self):
+        """Load the palette from the configuration into the color buttons."""
+        colourpalette = self.config['palette'].split(':')
+        for palette_id in range(0, NUM_PALETTE_COLORS):
+            color = Gdk.color_parse(colourpalette[palette_id])
+            self.load_palette_color(palette_id, color)
+
+    def edit_palette_button(self, widget):
+        """When the palette colorbutton is clicked, open a dialog to
+        configure a custom color."""
+        terminator = Terminator()
+        palette_id = self.get_palette_id(widget)
+        orig = self.get_palette_color(palette_id)
+
+        try:
+            # Create the dialog to choose a custom color
+            dialog = Gtk.ColorChooserDialog("Choose Palette Color")
+            dialog.set_rgba(Gdk.RGBA.from_color(orig))
+
+            def on_color_set(_, color):
+                # The color is set, so save the palette config and refresh Terminator
+                self.replace_palette_color(palette_id, dialog.get_rgba().to_color())
+                terminator.reconfigure()
+            dialog.connect('notify::rgba', on_color_set)
+
+            # Show the dialog
+            res = dialog.run()
+            if res != Gtk.ResponseType.OK:
+                # User cancelled the color change, so reset to the original.
+                self.replace_palette_color(palette_id, orig)
+                terminator.reconfigure()
+        finally:
+            if dialog:
+                dialog.destroy()
+
     def on_exit_action_combobox_changed(self, widget):
         """Exit action changed"""
         selected = widget.get_active()
@@ -1071,6 +1250,18 @@ class PrefsEditor:
         label_widget = guiget('handlesize_value_label')
         label_widget.set_text(str(value))
 
+    def on_lineheight_value_changed(self, widget):
+        """Handles line height changed"""
+        value = widget.get_value()
+        value = round(float(value), 1)
+        if value > 2.0:
+            value = 2.0
+        self.config['line_height'] = value
+        self.config.save()
+        guiget = self.builder.get_object
+        label_widget = guiget('lineheight_value_label')
+        label_widget.set_text(str(value))
+
     def on_focuscombo_changed(self, widget):
         """Focus type changed"""
         selected = widget.get_active()
@@ -1207,6 +1398,7 @@ class PrefsEditor:
         if self.config.replace_layout(name, current_layout):
             treeview.set_cursor(model.get_path(rowiter), column=treeview.get_column(0), start_editing=False)
         self.config.save()
+        self.layouteditor.set_layout(name)
 
     def on_layoutremovebutton_clicked(self, _button):
         """Remove a layout from the list"""
@@ -1307,13 +1499,21 @@ class PrefsEditor:
         backtype = None
         imagewidget = guiget('image_radiobutton')
         transwidget = guiget('transparent_radiobutton')
-        if transwidget.get_active() == True:
+
+        if imagewidget.get_active() == True:
+            backtype = 'image'
+        elif transwidget.get_active() == True:
             backtype = 'transparent'
         else:
             backtype = 'solid'
         self.config['background_type'] = backtype
         self.config.save()
 
+        if backtype == 'image':
+                guiget('background_image_file').set_sensitive(True)
+        else:
+                guiget('background_image_file').set_sensitive(False)
+
         if backtype in ('transparent', 'image'):
             guiget('darken_background_scale').set_sensitive(True)
         else:
@@ -1444,12 +1644,12 @@ class PrefsEditor:
         guiget = self.builder.get_object
         active = widget.get_active()
 
-        for key in self.colorschemevalues.keys():
+        for key in list(self.colorschemevalues.keys()):
             if self.colorschemevalues[key] == active:
                 value = key
 
-        fore = guiget('foreground_colorpicker')
-        back = guiget('background_colorpicker')
+        fore = guiget('foreground_colorbutton')
+        back = guiget('background_colorbutton')
         if value == 'custom':
             fore.set_sensitive(True)
             back.set_sensitive(True)
@@ -1462,18 +1662,8 @@ class PrefsEditor:
         if value in self.colourschemes:
             forecol = self.colourschemes[value][0]
             backcol = self.colourschemes[value][1]
-        elif value == 'custom':
-            forecol = color2hex(fore)
-            backcol = color2hex(back)
-        else:
-            err('Unknown colourscheme value: %s' % value)
-            return
-
-        fore.set_color(Gdk.color_parse(forecol))
-        back.set_color(Gdk.color_parse(backcol))
-
-        self.config['foreground_color'] = forecol
-        self.config['background_color'] = backcol
+            self.config['foreground_color'] = forecol
+            self.config['background_color'] = backcol
         self.config.save()
 
     def on_use_theme_colors_checkbutton_toggled(self, widget):
@@ -1482,8 +1672,8 @@ class PrefsEditor:
         active = widget.get_active()
 
         scheme = guiget('color_scheme_combobox')
-        fore = guiget('foreground_colorpicker')
-        back = guiget('background_colorpicker')
+        fore = guiget('foreground_colorbutton')
+        back = guiget('background_colorbutton')
 
         if active:
             for widget in [scheme, fore, back]:
@@ -1495,8 +1685,70 @@ class PrefsEditor:
         self.config['use_theme_colors'] = active
         self.config.save()
 
+    def on_bold_text_is_bright_checkbutton_toggled(self, widget):
+        """Bold-is-bright setting changed"""
+        self.config['bold_is_bright'] = widget.get_active()
+        self.config.save()
+
     def on_cellrenderer_accel_edited(self, liststore, path, key, mods, _code):
         """Handle an edited keybinding"""
+        # Ignore `Gdk.KEY_Tab` so that `Shift+Tab` is displayed as `Shift+Tab`
+        # in `Preferences>Keybindings` and NOT `Left Tab` (see `Gdk.KEY_ISO_Left_Tab`).
+        if mods & Gdk.ModifierType.SHIFT_MASK and key != Gdk.KEY_Tab:
+            key_with_shift = Gdk.Keymap.translate_keyboard_state(
+                self.keybindings.keymap,
+                hardware_keycode=_code,
+                state=Gdk.ModifierType.SHIFT_MASK,
+                group=0,
+            )
+            keyval_lower, keyval_upper = Gdk.keyval_convert_case(key)
+
+            # Remove the Shift modifier from `mods` if a new key binding doesn't
+            # contain a letter and its key value (`key`) can't be modified by a
+            # Shift key.
+            if key_with_shift.level != 0 and keyval_lower == keyval_upper:
+                mods = Gdk.ModifierType(mods & ~Gdk.ModifierType.SHIFT_MASK)
+                key = key_with_shift.keyval
+
+        accel = Gtk.accelerator_name(key, mods)
+        current_binding = liststore.get_value(liststore.get_iter(path), 0)
+
+        duplicate_bindings = []
+        for conf_binding, conf_accel in self.config["keybindings"].items():
+            parsed_accel = Gtk.accelerator_parse(accel)
+            parsed_conf_accel = Gtk.accelerator_parse(conf_accel)
+
+            if (
+                parsed_accel == parsed_conf_accel
+                and current_binding != conf_binding
+            ):
+                duplicate_bindings.append((conf_binding, conf_accel))
+
+        if duplicate_bindings:
+            dialog = Gtk.MessageDialog(
+                transient_for=self.window,
+                flags=Gtk.DialogFlags.MODAL,
+                message_type=Gtk.MessageType.ERROR,
+                buttons=Gtk.ButtonsType.CLOSE,
+                text="Duplicate Key Bindings Are Not Allowed",
+            )
+
+            accel_label = Gtk.accelerator_get_label(key, mods)
+            # get the first found duplicate
+            duplicate_keybinding_name = duplicate_bindings[0][0]
+
+            message = (
+                "Key binding `{0}` is already in use to trigger the `{1}` action."
+            ).format(accel_label, self.keybindingnames[duplicate_keybinding_name])
+            dialog.format_secondary_text(message)
+
+            self.active_message_dialog = dialog
+            dialog.run()
+            dialog.destroy()
+            self.active_message_dialog = None
+
+            return
+
         celliter = liststore.get_iter_from_string(path)
         liststore.set(celliter, 2, key, 3, mods)
 
@@ -1511,7 +1763,7 @@ class PrefsEditor:
         liststore.set(celliter, 2, 0, 3, 0)
 
         binding = liststore.get_value(liststore.get_iter(path), 0)
-        self.config['keybindings'][binding] = None
+        self.config['keybindings'][binding] = ""
         self.config.save()
 
     def on_open_manual(self,  widget):
@@ -1549,7 +1801,7 @@ class LayoutEditor:
         listitems = {}
         store.clear()
 
-        children = layout.keys()
+        children = list(layout.keys())
         i = 0
         while children != []:
             child = children.pop()
@@ -1645,17 +1897,17 @@ class LayoutEditor:
         command.set_sensitive(True)
         chooser.set_sensitive(True)
         workdir.set_sensitive(True)
-        if layout_item.has_key('command') and layout_item['command'] != '':
+        if 'command' in layout_item and layout_item['command'] != '':
             command.set_text(layout_item['command'])
         else:
             command.set_text('')
 
-        if layout_item.has_key('profile') and layout_item['profile'] != '':
+        if 'profile' in layout_item and layout_item['profile'] != '':
             chooser.set_active(self.profile_profile_to_ids[layout_item['profile']])
         else:
             chooser.set_active(0)
 
-        if layout_item.has_key('directory') and layout_item['directory'] != '':
+        if 'directory' in layout_item and layout_item['directory'] != '':
             workdir.set_text(layout_item['directory'])
         else:
             workdir.set_text('')
@@ -1684,9 +1936,9 @@ class LayoutEditor:
         self.config.save()
 
 if __name__ == '__main__':
-    import util
+    from . import util
     util.DEBUG = True
-    import terminal
+    from . import terminal
     TERM = terminal.Terminal()
     PREFEDIT = PrefsEditor(TERM)
 
