@@ -1,4 +1,3 @@
-#!/usr/bin/env python2
 # Terminator by Chris Jones <cmsj@tenshu.net>
 # GPL v2 only
 """window.py - class for the main Terminator window"""
@@ -8,16 +7,22 @@ import time
 import uuid
 import gi
 from gi.repository import GObject
-from gi.repository import Gtk, Gdk, GdkX11
+from gi.repository import Gtk, Gdk
 
-from util import dbg, err, make_uuid, display_manager
-import util
-from translation import _
-from version import APP_NAME
-from container import Container
-from factory import Factory
-from terminator import Terminator
+from .util import dbg, err, make_uuid, display_manager
 
+try:
+    from gi.repository import GdkX11
+except ImportError:
+    dbg("could not import X11 gir module")
+
+
+from . import util
+from .translation import _
+from .version import APP_NAME
+from .container import Container
+from .factory import Factory
+from .terminator import Terminator
 if display_manager() == 'X11':
     try:
         gi.require_version('Keybinder', '3.0')
@@ -42,6 +47,7 @@ class Window(Container, Gtk.Window):
     ignore_startup_show = None
     set_pos_by_ratio = None
     last_active_term = None
+    preventHide = None
 
     zoom_data = None
 
@@ -74,6 +80,8 @@ class Window(Container, Gtk.Window):
 
         self.title = WindowTitle(self)
         self.title.update()
+        
+        self.preventHide = False
 
         options = self.config.options_get()
         if options:
@@ -83,9 +91,6 @@ class Window(Container, Gtk.Window):
             if options.role:
                 self.set_role(options.role)
             
-#            if options.classname is not None:
-#                self.set_wmclass(options.classname, self.wmclass_class)
-            
             if options.forcedicon is not None:
                 icon_to_apply = options.forcedicon
 
@@ -229,9 +234,13 @@ class Window(Container, Gtk.Window):
             terminal.on_window_focus_out()
 
         self.losefocus_time = time.time()
-        if self.config['hide_on_lose_focus'] and self.get_property('visible'):
-            self.position = self.get_position()
-            self.hidefunc()
+
+        if self.preventHide:
+            self.preventHide = False
+        else:
+            if self.config['hide_on_lose_focus'] and self.get_property('visible'):
+                self.position = self.get_position()
+                self.hidefunc()
 
     def on_focus_in(self, window, event):
         """Focus has entered the window"""
@@ -833,9 +842,9 @@ class Window(Container, Gtk.Window):
             for term in possibles:
                 rect = layout[term]
                 offsets[term] = util.get_nav_offset(edge, rect, direction)
-            keys = offsets.values()
+            keys = list(offsets.values())
             keys.sort()
-            winners = [k for k, v in offsets.iteritems() if v == keys[0]]
+            winners = [k for k, v in offsets.items() if v == keys[0]]
             next = terminals.index(winners[0])
 
             if len(winners) > 1:
@@ -858,7 +867,7 @@ class Window(Container, Gtk.Window):
 
     def create_layout(self, layout):
         """Apply any config items from our layout"""
-        if not layout.has_key('children'):
+        if 'children' not in layout:
             err('layout describes no children: %s' % layout)
             return
         children = layout['children']
@@ -867,7 +876,7 @@ class Window(Container, Gtk.Window):
             err('incorrect number of children for Window: %s' % layout)
             return
 
-        child = children[children.keys()[0]]
+        child = children[list(children.keys())[0]]
         terminal = self.get_children()[0]
         dbg('Making a child of type: %s' % child['type'])
         if child['type'] == 'VPaned':
@@ -888,10 +897,10 @@ class Window(Container, Gtk.Window):
 
         self.get_children()[0].create_layout(child)
 
-        if layout.has_key('last_active_term') and layout['last_active_term'] not in ['', None]:
+        if 'last_active_term' in layout and layout['last_active_term'] not in ['', None]:
             self.last_active_term = make_uuid(layout['last_active_term'])
 
-        if layout.has_key('last_active_window') and layout['last_active_window'] == 'True':
+        if 'last_active_window' in layout and layout['last_active_window'] == 'True':
             self.terminator.last_active_window = self.uuid
 
 class WindowTitle(object):
